function [A derivatives] = ...
    Afunction(h, delta_x, equivmu, equivK, Sigma, maturity)
% Afunction constructs A_\tau and its derivatives. 
%
% This is a yield mapping for the purposes of this model.  The
% underlying model is Gaussian, with the restriction that delta_0 = 0.
% In other words, there is no constant term in the short rate; it
% is a linear function of the state vector x, or r_t = delta_x'
% x_t.  The equivalent martingale dynamics of x_t are
%
% x_t = equivmu + equivK *x_{t-1} + Sigma * shocks.  
%
% The derivatives are returned in a vector, where
% the dimension of the vector indexes the parameter.  
% The ordering of the parameters is:
% delta_x (n)
% equivmu (n)
% vec(equivK) (n * n)  
% the elements of Sigma, ordered row by row.  (n*(n+1)/2)
%
% Greg Duffee
% August 2006
%

n =  length(delta_x);
ident = eye(n); 

if (maturity == 1) 
  A = 0;
else
  inverse_term = inv(ident - equivK);
  power_term = equivK^maturity;
  Q = Sigma * Sigma'; 
  A = (1/maturity)*delta_x'*(maturity*ident - inverse_term* ...
                        (ident - power_term))*inverse_term*equivmu; 
  sumB = 0; 
  for i = 1:(maturity-1)
    this_bprime = Bfunction(delta_x, equivK, i); 
    sumB = sumB + (h/(2*maturity))*i^2*this_bprime*Q*this_bprime'; 
  end
  A = A - sumB;
end
% 
% should derivatives be calculated?
%
if (nargout > 1) 
  derivatives = zeros( (2*n+n^2+(n*(n+1)/2)), 1);
  if (maturity > 1) 
    % generate the derivatives of B_i wrt all of the parameters, for
    % use below.  k runs from 1 to (maturity - 1).  The 3rd index
    % indicates the maturity k, the second indicates the parameter of
    % Bfunction, and the first is the derivative of element i of B_i.
    b_i_vals = zeros(n, (maturity - 1)); 
    b_i_derivs = zeros(n, (n+n^2), (maturity - 1)); 
    for i = 1:(maturity - 1)
      [this_bval this_deriv] = Bfunction(delta_x, equivK, i);
      b_i_vals(:,i) = this_bval';
      b_i_derivs(:,:,i) = this_deriv; 
    end
    %
    % derivative wrt delta_x
    %
    for i = 1:n
      Z = zeros(n,1);
      Z(i) = 1;
      deriv = (1/maturity)*Z'*(maturity*ident - inverse_term* ...
                               (ident - power_term))*inverse_term*equivmu; 
      sumB = 0; 
      for k = 1:(maturity-1)
        this_bval = b_i_vals(:,k);
        this_deriv = b_i_derivs(:,i,k); 
        sumB = sumB + (h/(2*maturity))*k^2*( ...
            this_deriv'*Q*this_bval + this_bval'*Q*this_deriv); 
      end
      derivatives(i) = deriv - sumB; % these are in spots 1 to n
    end
    %
    % derivative wrt equivmu
    %
    for i = 1:n
      Z = zeros(n,1);
      Z(i) = 1;
      deriv = (1/maturity)*delta_x'*(maturity*ident - inverse_term* ...
                                     (ident - power_term))*inverse_term*Z; 
      derivatives(n+i) = deriv; % these are in spots n+1 to 2n
    end
    %
    % derivative wrt vec(K) 
    %
    index = 2*n;  % tracks the index of this derivative
    alt_index = n;  % corresponds to index of B derivative
    for j = 1:n
      for i = 1:n
        index = index + 1; 
        alt_index = alt_index + 1; 
        Z=zeros(n,n);
        Z(i,j) = 1; 
        powerderiv_term = 0;
        for k = 1:maturity
          powerderiv_term = powerderiv_term + ...
              equivK^(k-1)*Z*equivK^(maturity-k);
        end
        first_term = powerderiv_term - Z*inverse_term*(ident - ...
                                                       power_term); 
        second_term = maturity*ident - inverse_term*(ident - power_term);
        third_term = second_term*inverse_term*Z; 
        deriv = (1/maturity)*delta_x'* ...
                (inverse_term*first_term + third_term)*inverse_term*equivmu; 
        sumB = 0; 
        for k = 1:(maturity-1)
          this_bval = b_i_vals(:,k);
          this_deriv = b_i_derivs(:,alt_index,k); 
          sumB = sumB + (h/(2*maturity))*k^2*( ...
              this_deriv'*Q*this_bval + this_bval'*Q*this_deriv); 
        end
        derivatives(index) = deriv - sumB; % spots 2n+2 to 2n+1+n^2
      end
    end
    %
    % derivative wrt sigma  
    %
    index = 2*n+n^2;
    for i = 1:n
      for j = 1:i
        index = index + 1; 
        Z = zeros(n,n);
        Z(i,j) = 1; 
        sumB = 0;
        for k = 1:(maturity - 1)
          this_bval = b_i_vals(:,k);
          sumB = sumB + (h/(2*maturity))*k^2*this_bval'*( ...
              Z*Sigma' + Sigma*Z')*this_bval; 
          derivatives(index) = -sumB; 
        end
      end
    end
  end
end

return
