function [Bprime derivatives] = Bfunction(delta_x, equivK, maturity)
% Bfunction constructs the transpose of B_\tau and its derivatives. 
%
% This is a yield mapping for the purposes of this model.  The
% underlying model is Gaussian, with the restriction that delta_0 = 0.
% In other words, there is no constant term in the short rate; it
% is a linear function of the state vector x, or r_t = delta_x'
% x_t.  The equivalent martingale dynamics of x_t are
%
% x_t = equivmu + equivK *x_{t-1} + Sigma * shocks.  
%
% The derivatives are returned in a 2-d array.  Element (i,j) is
% the derivative of the i'th element of Bprime with respect to
% parameter j.  
%
% The ordering of the parameters is:
% delta_x (n)
% vec(equivK) (n * n)  
%
% Greg Duffee
% August 2006
%

n = length(delta_x); 
ident = eye(n); 

if (maturity == 1)
  Bprime = delta_x';
else
  inverse_term = inv(ident - equivK); 
  power_term = equivK^maturity; 
  Bprime = (1/maturity)*delta_x'*inverse_term*(ident - power_term);
end
% 
% should derivatives be calculated?
%
if (nargout > 1) 
  derivatives = zeros(n, (n+n^2));
  if (maturity == 1)
    derivatives(1:n,1:n) = ident;
  else
    % derivatives with respect to elements of delta_x
    for i = 1:n
      Z=zeros(n,1);
      Z(i) = 1;
      derivatives(:,i) = ...
          ((1/maturity)*Z'*inverse_term*(ident - power_term))';
    end
    % derivatives with respect to elements of K
    index = n;
    for j = 1:n
      for i = 1:n
        index = index + 1; 
        Z=zeros(n,n);
        Z(i,j) = 1; 
        first_term = inverse_term*Z*inverse_term*(ident - power_term); 
        second_term = 0;
        for k = 1:maturity
          second_term = second_term + equivK^(k-1)*Z*equivK^(maturity-k);
        end
        third_term = -inverse_term*second_term;
        derivatives(:,index) = ...
            ((1/maturity)*delta_x'*(first_term + third_term))';
      end
    end
  end
end

return
