% Greg Duffee
% Latest revision June 2007
%

%------------------------------------------------------------
%
% This routine reads actual data and puts it in a form suitable
% for use by the other Matlab routines.  
%
% The data are inflation (GDP price index), real GDP growth, 
% the three-month T-bill yield, and zero-coupon bond yields from
% the Fed.  
%
% This code writes ASCII data files containing "macro" data, which
% means the short rate, inflation, and output growth, and "yield"
% data, which means multiperiod bond yields. 
%
% The data span 1952Q2 through 2005Q4.  This is done mostly for
% historical reasons.  The Fed yield data begin in 1961Q2, so there
% are many missing values in these yield data.  
%
%------------------------------------------------------------
%
% 1.  Read "macro" data from files
%
%------------------------------------------------------------

clear

% GDP price index, 1947Q1 through 2006Q1

fid=fopen('..\matlabData\gdp_price_index.dat'); 
data = textscan(fid, '%d %f'); 
gdp_price_index = data{2}; 
fclose(fid); 

% real GDP, 1947Q1 through 2006Q1

fid=fopen('..\matlabData\gdp_real.dat'); 
data = textscan(fid, '%d %f'); 
real_gdp = data{2}; 
fclose(fid); 

% 3-month T-bill yield, 1925Q4 through 2005Q4

fid=fopen('..\matlabData\crsp_riskfree_q.dat'); 
data = textscan(fid, '%d %f %f', -1); 
raw_shortrate = data{3}; 
fclose(fid); 

%------------------------------------------------------------
%
% 2.  Transform data to variables used in the paper -- 
%     inflation, output growth, short rate.  
%
%------------------------------------------------------------

raw_pi =  400*log(gdp_price_index(2:end) ./gdp_price_index(1:(end-1)));
raw_d_gdp = 400*(log(real_gdp(2:end)) - log(real_gdp(1:(end-1))));

% All data begin with 1952Q2.  Therefore drop 20 quarters (5 years)
% of data from pi and d_gdp.  Drop 21 quarters (5 years + 1 quarter) from
% shortrate.  All data end with 2005Q4.  Therefore drop last quarter of 
% data from pi and d_gdp. 
%

raw_pi = raw_pi(21:(end-1));
raw_d_gdp = raw_d_gdp(21:(end-1)); 
raw_shortrate = raw_shortrate(107:end); 

pi = raw_pi/100.0;
d_gdp = raw_d_gdp/100.0;
shortrate = raw_shortrate/100.0; 

%------------------------------------------------------------
%
% note -- I am putting the short rate in the third position. 
%
%------------------------------------------------------------

data_matrix = [pi d_gdp shortrate]; 

%------------------------------------------------------------
%
% 3.  Write macro output 
%
%------------------------------------------------------------

dlmwrite('..\matlabData\macrodata.dat', data_matrix, ...
      'delimiter', ' ', 'newline', 'pc', 'precision', 6);

%------------------------------------------------------------
%
% 4.  Read Fed zero data from excel spread sheet.
%
%------------------------------------------------------------

filename = char('..\matlabData\feds200628.xls'); 
sheetname = char('gswdata'); 

[data headings] = xlsread(filename, sheetname);
year = data(:,1);
month = data(:,2); 
day = data(:,3); 
q1_end = (month(1:(end-1)) == 3) & (month(1:(end-1)) ~= month(2: ...
                                                  end));
q2_end = (month(1:(end-1)) == 6) & (month(1:(end-1)) ~= month(2: ...
                                                  end));
q3_end = (month(1:(end-1)) == 9) & (month(1:(end-1)) ~= month(2: ...
                                                  end));
q4_end = (month(1:(end-1)) == 12) & (month(1:(end-1)) ~= month(2: ...
                                                  end));
short_data = data(1:(end-1),:);
unsorted_data = [short_data(q1_end,:); short_data(q2_end,:); ...
              short_data(q3_end,:); short_data(q4_end,:)];
qend_data = sortrows(unsorted_data, [1; 2; 3]);
qend_dates = qend_data(:,1:3);
qend_rates = qend_data(:,4:end); 

% the version of the Fed quarter-end data that I have ends with 2006Q2,
% so I drop the last two observations.  

qend_dates = qend_dates(1:(end-2),:);
qend_rates = qend_rates(1:(end-2),:); 

% I construct three files.  One contains yields on maturities of one,
% three, five, and seven years.  Another contains yields on
% maturities of one, three, five, seven, and ten years.  The final
% file contains yields on maturities of one through seven years. 
% The files have to be padded with missing value codes (-99) to
% line up with the sizes of the other files.  


max7_indexes = [1;2;3;5;7];
max10_indexes = [1;2;3;5;7;10];
all7_indexes = [1;2;3;4;5;6;7]; 
padding_7 = repmat(-99, (size(data_matrix,1) - size(qend_rates,1)), ...
                   length(max7_indexes)); 
padding_10 = repmat(-99, (size(data_matrix,1) - size(qend_rates,1)), ...
                   length(max10_indexes)); 
padding_all7 = repmat(-99, (size(data_matrix,1) - size(qend_rates,1)), ...
                   length(all7_indexes)); 
max7_rates = [padding_7; qend_rates(:,max7_indexes)/100];
max10_rates = [padding_10; qend_rates(:,max10_indexes)/100];
max10_rates(isnan(max10_rates)) = -99.0; 
all7_rates = [padding_all7; qend_rates(:,all7_indexes)/100];

%------------------------------------------------------------
%
% Write output files. 
%
%------------------------------------------------------------

dlmwrite('..\matlabData\yield7data.dat', max7_rates, ...
      'delimiter', ' ', 'newline', 'pc', 'precision', 6);

dlmwrite('..\matlabData\allyield7data.dat', all7_rates, ...
      'delimiter', ' ', 'newline', 'pc', 'precision', 6);

dlmwrite('..\matlabData\yield10data.dat', max10_rates, ...
      'delimiter', ' ', 'newline', 'pc', 'precision', 6);

dlmwrite('..\matlabData\mats_max7.dat', 4*max7_indexes, ...
      'delimiter', ' ', 'newline', 'pc', 'precision', 6);

dlmwrite('..\matlabData\mats_max10.dat', 4*max10_indexes, ...
      'delimiter', ' ', 'newline', 'pc', 'precision', 6);

