% Greg Duffee
% April 2007
% 
%
% Run Monte Carlo simulations for in-sample and out-of-sample
% regressions. 
%

clear

useLate = 1; 

if useLate 
  outputfile = char('..\matlabFiles\late_onlyomega3.out'); 
else
  outputfile = char('..\matlabFiles\full_onlyomega2.out'); 
end

fprintf(1, '\n\n'); 
fprintf(1, 'File containing true parameters is %s.\n',outputfile);

%--------------------------------------------------------
%
% Choose the desired macro variables used in the univariate
% regressions.  1=inflation, 2=output growth, 3 = short rate.  Any
% combination of these variables can be used here.  
%
%--------------------------------------------------------

used_macro_indexes = [1 2 3]; 

fprintf(1, 'Explanatory variable(s):');
if (sum(used_macro_indexes == 1) == 1)
  fprintf(1, '  Inflation');
end
if (sum(used_macro_indexes == 2) == 1)
  fprintf(1, '  Output growth');
end
if (sum(used_macro_indexes == 3) == 1)
  fprintf(1, '  Short rate');
end
fprintf(1, '\n');

%--------------------------------------------------------
%
% Choose the original maturity of the desired bond.  This
% maturity is in quarters.
%
%--------------------------------------------------------

init_mat = 8; 
%init_mat = 28; 

fprintf(1, 'Original maturity (quarters) = %d.\n', init_mat); 

%--------------------------------------------------------
%
% What return horizons are used for simple returns?  (these are
% measured in quarters) For a given horizon, what MA lags are used
% in computing standard errors?  The rows of 'simple_ma_lag_matrix'
% correspond to the different return horizons.  Are newey-west or
% generalized hansen-hodrick standard errors used?  If
% 'simple_newey_west' contains a one, newey-west standard errors
% are used for that combination of horizon and MA lags.  
%
%--------------------------------------------------------

simple_horizons = [1];
simple_ma_lag_matrix = [0 4]; 
simple_newey_west = ones(size(simple_ma_lag_matrix)); 

%--------------------------------------------------------
%
% Recall that for log returns, only a four-quarter horizon is
% considered.  For this horizon, what MA lags are used
% in computing standard errors?  The elements of
% 'log_ma_lag_vector' contain these different lags.  Are newey-west or
% generalized hansen-hodrick standard errors used?  If
% 'log_newey_west' contains a one, newey-west standard errors
% are used for that MA lag.  
%
%--------------------------------------------------------

log_ma_lag_vector = [3 7]; 
log_newey_west = ones(size(log_ma_lag_vector)); 
log_newey_west(1) = 0; 

%--------------------------------------------------------
%
% 6.  Information about the simulations.  How many?  What is the
% length of a single simulation?  What is the initial seed for the
% random number generator? ** and ** here is where the length of
% the initial regression is set for the out-of-sample tests.  If
% this length is zero, no out-of-sample tests are performed.  
%
%--------------------------------------------------------

init_simseed = 123456; 
numsims = 5000; 

fprintf(1, 'Number of simulations is %d.\n', numsims);

% use below to simulate full sample period.  

sim_length = 179; 
insamp_length = 100;

% use below to simulate later sample period.  

%sim_length = 84; 
%insamp_length = 0; 

fprintf(1, 'Length of simulated data series is %d.\n', sim_length);
fprintf(1, ['Initial length of regression for ' ...
            'out-of-sample test is %d.\n'], insamp_length);

%--------------------------------------------------------
%
% For the first set of results, turn off the latent factors -- this
% is the restrictive null. 
%
%--------------------------------------------------------

shutoff_latent = 1;

%-------------------------------------------------------
% 
% Run simulations
%
%------------------------------------------------------

[simple_insamp_results simple_outsamp_results ...
         log_insamp_results log_outsamp_results] = ...
    sim_regressions( ...
    outputfile, shutoff_latent, numsims, sim_length, init_simseed, ...
    used_macro_indexes, simple_newey_west, log_newey_west, ...
    init_mat, simple_horizons, simple_ma_lag_matrix, ...
    log_ma_lag_vector, insamp_length);

%-------------------------------------------------------
% 
% Summarize the results.
%
%------------------------------------------------------

this_horiz = 1; % index of relevant horizon for simple returns
used_ma_lags = simple_ma_lag_matrix(this_horiz,:); 
used_newey_west = simple_newey_west(this_horiz,:); 
num_simple_ma_lags = size(simple_ma_lag_matrix,2); 
num_log_ma_lags = length(log_ma_lag_vector); 
num_macro = length(used_macro_indexes); 

[stacked_simple_results, insample_simple_test_info, ...
          out_of_samp_simple_summary, outsamp_simple_stat_info, ...
          stacked_log_results, insamp_log_test_info, ...
          out_of_samp_log_summary, outsamp_log_stat_info] = ...
    summarize_reg_sims(this_horiz, num_macro, ...
             simple_insamp_results, simple_outsamp_results, ...
             log_insamp_results, log_outsamp_results, ...
             simple_ma_lag_matrix, log_ma_lag_vector); 


fprintf(1, ['\nStandard macro-finance model (no term premia ' ...
            'factors)\n\n']);

fprintf(1, 'Results for simple returns\n');
fprintf(1, '--------------------------\n\n'); 
fprintf(1, 'Holding period of simple return is %d quarters.\n', ...
        simple_horizons(this_horiz)); 
fprintf(1, '\n\n'); 

fprintf(1, '             Constant ')
for i = 1:num_macro
  fprintf(1, '      X%d   ', i);
end
fprintf(1, '\n'); 
fprintf(1, 'Mean coef  %9.4f', stacked_simple_results(1,1)); 
for i = 1:num_macro
  fprintf(1, ' %9.4f ', stacked_simple_results(1,(1+i)));
end
fprintf(1, '\n'); 
fprintf(1, 'Std dev    %9.4f', stacked_simple_results(2,1)); 
for i = 1:num_macro
  fprintf(1, ' %9.4f ', stacked_simple_results(2,(1+i)));
end
fprintf(1, '\n\n'); 
fprintf(1, '        Mean standard errors\n'); 
fprintf(1, '  ----------------------------------\n\n'); 
fprintf(1, 'MA lags     NW?    Constant  ')
for i = 1:num_macro
  fprintf(1, '      X%d   ', i);
end
fprintf(1, '\n'); 
for j = 1:length(used_ma_lags)
  fprintf(1, '    %d        %d    %9.4f', ...
          used_ma_lags(j), used_newey_west(j), ...
          stacked_simple_results(2+j,1)); 
  for i = 1:num_macro
    fprintf(1, ' %9.4f ', stacked_simple_results(2+j,(1+i)));
  end
  fprintf(1, '\n'); 
end

fprintf(1, '\n'); 
fprintf(1, 'Joint test that macro coefs are all zero\n'); 
fprintf(1, '----------------------------------------\n\n'); 
fprintf(1, [' MA   Use   emp reject  true 5%%  asmp reject ' ...
            'rate\n']);
fprintf(1, ['lags  NW?      at 5%%    critval  with this ' ...
            'critval\n']);
for i = 1:length(used_ma_lags)
  fprintf(1, '  %d   %d    %9.4f %9.4f    %9.4f\n', used_ma_lags(i), ...
          used_newey_west(i), insample_simple_test_info(i,1), ...
          insample_simple_test_info(i,2), ...
          insample_simple_test_info(i,3));
end
fprintf(1, '\n\n'); 

if (insamp_length > 0) 

  fprintf(1, 'Out of sample test\n'); 
  fprintf(1, '------------------\n\n'); 
  fprintf(1, '               Mean     Std dev     emp reject  true 5%%\n'); 
  fprintf(1, 'MA lags NW?   t-stat   of t-stat       at 5%%   crit val\n'); 
  for i = 1:length(used_ma_lags)
    fprintf(1, '    %d    %d  %9.4f  %9.4f  %9.4f %9.4f\n', ...
            used_ma_lags(i), used_newey_west(i), ...
            out_of_samp_simple_summary(1,i), ...
            out_of_samp_simple_summary(2,i), ...
            outsamp_simple_stat_info(i,1), ...
            outsamp_simple_stat_info(i,2));
  end

end

fprintf(1, 'Results for log annual returns\n');
fprintf(1, '------------------------------\n\n'); 

fprintf(1, '             Constant ')
for i = 1:num_macro
  fprintf(1, '      X%d   ', i);
end
fprintf(1, '\n'); 
fprintf(1, 'Mean coef  %9.4f', stacked_log_results(1,1)); 
for i = 1:num_macro
  fprintf(1, ' %9.4f ', stacked_log_results(1,(1+i)));
end
fprintf(1, '\n'); 
fprintf(1, 'Std dev    %9.4f', stacked_log_results(2,1)); 
for i = 1:num_macro
  fprintf(1, ' %9.4f ', stacked_log_results(2,(1+i)));
end
fprintf(1, '\n\n'); 
fprintf(1, '        Mean standard errors\n'); 
fprintf(1, '  ----------------------------------\n\n'); 
fprintf(1, 'MA lags     NW?    Constant  ')
for i = 1:num_macro
  fprintf(1, '      X%d   ', i);
end
fprintf(1, '\n'); 
for j = 1:length(used_ma_lags)
  fprintf(1, '    %d        %d    %9.4f', ...
          used_ma_lags(j), used_newey_west(j), ...
          stacked_log_results(2+j,1)); 
  for i = 1:num_macro
    fprintf(1, ' %9.4f ', stacked_log_results(2+j,(1+i)));
  end
  fprintf(1, '\n'); 
end

fprintf(1, '\n'); 
fprintf(1, 'Joint test that macro coefs are all zero\n'); 
fprintf(1, '----------------------------------------\n\n'); 
fprintf(1, [' MA   Use   emp reject  true 5%%\n']);
fprintf(1, ['lags  NW?      at 5%%    critval\n']);
for i = 1:num_log_ma_lags
  fprintf(1, '  %d   %d    %9.4f %9.4f\n', log_ma_lag_vector(i), ...
          log_newey_west(i), insamp_log_test_info(i,1), ...
          insamp_log_test_info(i,2))
end
fprintf(1, '\n\n'); 

if (insamp_length > 0) 

  fprintf(1, 'Out of sample test\n'); 
  fprintf(1, '------------------\n\n'); 
  fprintf(1, '               Mean     Std dev     emp reject  true 5%%\n'); 
  fprintf(1, 'MA lags NW?   t-stat   of t-stat       at 5%%   crit val\n'); 
  for i = 1:num_log_ma_lags
    fprintf(1, '    %d    %d  %9.4f  %9.4f  %9.4f %9.4f\n', ...
            log_ma_lag_vector(i), log_newey_west(i), ...
            out_of_samp_log_summary(1,i), ...
            out_of_samp_log_summary(2,i), ...
            outsamp_log_stat_info(i,1), ...
            outsamp_log_stat_info(i,2));
  end

end

%--------------------------------------------------------
%
% For the second set of results, use latent factors.
%
%--------------------------------------------------------

shutoff_latent = 0;

%-------------------------------------------------------
% 
% Run simulations
%
%------------------------------------------------------

[simple_insamp_results simple_outsamp_results ...
         log_insamp_results log_outsamp_results] = ...
    sim_regressions( ...
    outputfile, shutoff_latent, numsims, sim_length, init_simseed, ...
    used_macro_indexes, simple_newey_west, log_newey_west, ...
    init_mat, simple_horizons, simple_ma_lag_matrix, ...
    log_ma_lag_vector, insamp_length);

%-------------------------------------------------------
% 
% Summarize the results.
%
%------------------------------------------------------

this_horiz = 1; % index of relevant horizon for simple returns
used_ma_lags = simple_ma_lag_matrix(this_horiz,:); 
used_newey_west = simple_newey_west(this_horiz,:); 
num_simple_ma_lags = size(simple_ma_lag_matrix,2); 
num_log_ma_lags = length(log_ma_lag_vector); 
num_macro = length(used_macro_indexes); 

[stacked_simple_results, insample_simple_test_info, ...
          out_of_samp_simple_summary, outsamp_simple_stat_info, ...
          stacked_log_results, insamp_log_test_info, ...
          out_of_samp_log_summary, outsamp_log_stat_info] = ...
    summarize_reg_sims(this_horiz, num_macro, ...
             simple_insamp_results, simple_outsamp_results, ...
             log_insamp_results, log_outsamp_results, ...
             simple_ma_lag_matrix, log_ma_lag_vector); 


fprintf(1, ['\nGeneral null (includes term premia ' ...
            'factors)\n\n']);

fprintf(1, 'Results for simple returns\n');
fprintf(1, '--------------------------\n\n'); 
fprintf(1, 'Holding period of simple return is %d quarters.\n', ...
        simple_horizons(this_horiz)); 
fprintf(1, '\n\n'); 

fprintf(1, '             Constant ')
for i = 1:num_macro
  fprintf(1, '      X%d   ', i);
end
fprintf(1, '\n'); 
fprintf(1, 'Mean coef  %9.4f', stacked_simple_results(1,1)); 
for i = 1:num_macro
  fprintf(1, ' %9.4f ', stacked_simple_results(1,(1+i)));
end
fprintf(1, '\n'); 
fprintf(1, 'Std dev    %9.4f', stacked_simple_results(2,1)); 
for i = 1:num_macro
  fprintf(1, ' %9.4f ', stacked_simple_results(2,(1+i)));
end
fprintf(1, '\n\n'); 
fprintf(1, '        Mean standard errors\n'); 
fprintf(1, '  ----------------------------------\n\n'); 
fprintf(1, 'MA lags     NW?    Constant  ')
for i = 1:num_macro
  fprintf(1, '      X%d   ', i);
end
fprintf(1, '\n'); 
for j = 1:length(used_ma_lags)
  fprintf(1, '    %d        %d    %9.4f', ...
          used_ma_lags(j), used_newey_west(j), ...
          stacked_simple_results(2+j,1)); 
  for i = 1:num_macro
    fprintf(1, ' %9.4f ', stacked_simple_results(2+j,(1+i)));
  end
  fprintf(1, '\n'); 
end

fprintf(1, '\n'); 
fprintf(1, 'Joint test that macro coefs are all zero\n'); 
fprintf(1, '----------------------------------------\n\n'); 
fprintf(1, [' MA   Use   emp reject  true 5%%  asmp reject ' ...
            'rate\n']);
fprintf(1, ['lags  NW?      at 5%%    critval  with this ' ...
            'critval\n']);
for i = 1:length(used_ma_lags)
  fprintf(1, '  %d   %d    %9.4f %9.4f    %9.4f\n', used_ma_lags(i), ...
          used_newey_west(i), insample_simple_test_info(i,1), ...
          insample_simple_test_info(i,2), ...
          insample_simple_test_info(i,3));
end
fprintf(1, '\n\n'); 

if (insamp_length > 0) 

  fprintf(1, 'Out of sample test\n'); 
  fprintf(1, '------------------\n\n'); 
  fprintf(1, '               Mean     Std dev     emp reject  true 5%%\n'); 
  fprintf(1, 'MA lags NW?   t-stat   of t-stat       at 5%%   crit val\n'); 
  for i = 1:length(used_ma_lags)
    fprintf(1, '    %d    %d  %9.4f  %9.4f  %9.4f %9.4f\n', ...
            used_ma_lags(i), used_newey_west(i), ...
            out_of_samp_simple_summary(1,i), ...
            out_of_samp_simple_summary(2,i), ...
            outsamp_simple_stat_info(i,1), ...
            outsamp_simple_stat_info(i,2));
  end

end

fprintf(1, 'Results for log annual returns\n');
fprintf(1, '------------------------------\n\n'); 

fprintf(1, '             Constant ')
for i = 1:num_macro
  fprintf(1, '      X%d   ', i);
end
fprintf(1, '\n'); 
fprintf(1, 'Mean coef  %9.4f', stacked_log_results(1,1)); 
for i = 1:num_macro
  fprintf(1, ' %9.4f ', stacked_log_results(1,(1+i)));
end
fprintf(1, '\n'); 
fprintf(1, 'Std dev    %9.4f', stacked_log_results(2,1)); 
for i = 1:num_macro
  fprintf(1, ' %9.4f ', stacked_log_results(2,(1+i)));
end
fprintf(1, '\n\n'); 
fprintf(1, '        Mean standard errors\n'); 
fprintf(1, '  ----------------------------------\n\n'); 
fprintf(1, 'MA lags     NW?    Constant  ')
for i = 1:num_macro
  fprintf(1, '      X%d   ', i);
end
fprintf(1, '\n'); 
for j = 1:length(used_ma_lags)
  fprintf(1, '    %d        %d    %9.4f', ...
          used_ma_lags(j), used_newey_west(j), ...
          stacked_log_results(2+j,1)); 
  for i = 1:num_macro
    fprintf(1, ' %9.4f ', stacked_log_results(2+j,(1+i)));
  end
  fprintf(1, '\n'); 
end

fprintf(1, '\n'); 
fprintf(1, 'Joint test that macro coefs are all zero\n'); 
fprintf(1, '----------------------------------------\n\n'); 
fprintf(1, [' MA   Use   emp reject  true 5%%\n']);
fprintf(1, ['lags  NW?      at 5%%    critval\n']);
for i = 1:num_log_ma_lags
  fprintf(1, '  %d   %d    %9.4f %9.4f\n', log_ma_lag_vector(i), ...
          log_newey_west(i), insamp_log_test_info(i,1), ...
          insamp_log_test_info(i,2))
end
fprintf(1, '\n\n'); 

if (insamp_length > 0) 

  fprintf(1, 'Out of sample test\n'); 
  fprintf(1, '------------------\n\n'); 
  fprintf(1, '               Mean     Std dev     emp reject  true 5%%\n'); 
  fprintf(1, 'MA lags NW?   t-stat   of t-stat       at 5%%   crit val\n'); 
  for i = 1:num_log_ma_lags
    fprintf(1, '    %d    %d  %9.4f  %9.4f  %9.4f %9.4f\n', ...
            log_ma_lag_vector(i), log_newey_west(i), ...
            out_of_samp_log_summary(1,i), ...
            out_of_samp_log_summary(2,i), ...
            outsamp_log_stat_info(i,1), ...
            outsamp_log_stat_info(i,2));
  end

end

