function test_stats = enc_regtest(yvec, xmatrix, R, overlap_length, ...
                                 ma_lags, newey_west)
% 
% ENC-REG test of Ericsson, N.R., 1992, Parameter constancy, mean
% square forecast errors, and measuring forecast performance: an
% exposition, extensions, and illustration, Journal of Policy
% Modeling 14, 465-495.  The test is specialized to the
% case where the restricted model is simply a constant.  Therefore,
% under the null, demeaned returns are unforecastable.  The only
% adjustment to Ericsson is to account for overlapping observations
% in the regression of forecast errors on the cross-sectional
% difference of forecast errors.  The routine allows for multiple
% choices of MA lags in the calculation of the uncertainty in the
% parameter.  The choices of MA lags are in the vector 'ma_lags'
% and the flag 'newey_west' says whether to use newey_west standard
% errors or generalized hansen-hodrick standard errors.  
% 
% The total length of data is N.  The first R observations are used
% to form an initial set of parameter estimates to use in
% constructing a k-period-ahead forecast of Y.  Note that with k=1,
% there will be N-R-1 one-step-ahead forecasts for which a forecast
% error can be calculated.  (There are N-R forecasts, but we don't
% know the success of the final forecast.)  More generally, there
% are N-R-k k-step-ahead forecasts that we can construct.  
%
% The output is a vector of test statistics.  The length of the
% vector equals the length of 'ma_lags', with one test statistic
% per choice of MA lag length.  
%
% Greg Duffee
% February 2006, revised April 2006
%

u1 = []; u2 = []; 
N = length(yvec);
P = N - R - overlap_length; 

for i = R:(N-1-overlap_length)
  beta2 = regress(yvec(1:i), [ones(i,1) xmatrix(1:i,:)]);
  beta1 = mean(yvec(1:i));
  u2 = [u2; yvec(i+1+overlap_length) - ...
        beta2' * [1 xmatrix(i+1+overlap_length,:)]'];
  u1 = [u1; yvec(i+1+overlap_length) - beta1];
end
udiff = u1 - u2;

% run a regression of u1 on udiff.  Adjust for MA lags, if needed.  
% there is no constant term in the regression.  
%

T = size(u1,1); 
[coef temp resids] = regress(u1, udiff); 
moments = resids .* udiff;
test_stats = [];

XprimeX_inv = 1.0/(udiff' * udiff);
S0 = (1/T) * moments' * moments;
for lag_index = 1:length(ma_lags)
  lags = ma_lags(lag_index); 
  use_newey_west = newey_west(lag_index); 
  S = S0; 
  if (lags > 0) 
    for j = 1:lags
      F.matrix = (1/T) * transpose(moments(1:(end-j))) * moments((1+j):end); 
      if (use_newey_west == 1)
        S = S + (1.0 - (j/(lags+1.0))) * (F.matrix + transpose(F.matrix)); 
      else
        S = S + (F.matrix + transpose(F.matrix)); 
      end
    end
  end
  var = T * XprimeX_inv * S * XprimeX_inv; 
  if (var < 0.0) % not positive definite, then redo, using
                        % newey-west.  This is only relevant if
                        % lags > 0.  
    S = S0; 
    for j = 1:lags
      F.matrix = (1/T) * transpose(moments(1:(end-j))) * moments((1+j):end); 
      S = S + (1.0 - (j/(lags+1.0))) * (F.matrix + transpose(F.matrix)); 
    end
    var = T * XprimeX_inv * S * XprimeX_inv; 
  end
  test_stats = [test_stats; coef/sqrt(var)]; 
end

return
