function [numbonds all_estimates loglike score exitflag ...
         mean_hessian S_matrix T] = ...
    estimate_model(filename, printscreen, writetemp, ...
                   mineigen_allowed, kalman_tolerance, restrict_unconmean, ...
                   display_simplex, display_deriv, restart_index)
%
% ESTIMATE_MODEL : reads the control file 'filename' and estimates the
% term structure model.  Some intermediate results are printed to
% the screen if 'printscreen' == 1.  
%
% Greg Duffee
% November 2005, revised March 2006, Jan 2007  
%

% assume this is not a restart if there is no restart index. 
if (nargin<9); restart_index = 1; end;

% hardwire the number of macro factors
n = 3; 

[inputfile outputfile] = read_control(filename);
[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K_initialval fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S_initialval fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K_initialval prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S_initialval prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c_initialval prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro_initialval prem_macro_rand_stddev ...
        errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
       ] = read_input(inputfile, n); 

%
% don't bother writing temporary results if a single set of
% starting values is used.
%

writetemp_copy = writetemp; 
if (numloops < 2)
  writetemp_copy = 0;
end

[macrodata yielddata maturities] = read_data(macrodatafile, ...
          yielddatafile, maturitydatafile, numbonds, firstobs, ...
          lastobs); 

[freeparams allparams free_index] = setup_initparams( ...
    use_latent, fund_K_initialval, ...
    fund_S_initialval, prem_K_initialval, prem_S_initialval, ...
    prem_c_initialval, prem_macro_initialval, errorSD_initialval, ...
    fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag, macrodata, use_ols_start); 

stddevs = pack_allparams(...
    use_latent, fund_K_rand_stddev, ...
    fund_S_rand_stddev, prem_K_rand_stddev, prem_S_rand_stddev, ...
    prem_c_rand_stddev, prem_macro_rand_stddev, ...
    errorSD_rand_stddev, fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag); 

if (printscreen == 1) 

  fprintf(1, ['\n\nMaximum likelihood estimation of a macro ', ...
              'term structure model\n']);
  fprintf(1,     ['-----------------------------------------', ...
                  '--------------------\n\n']);
  fprintf(1, 'Input file is  %s.\n', inputfile);
  fprintf(1, 'Output file is %s.\n', outputfile);
  fprintf(1, 'Data files are:\n');
  fprintf(1, '%s\n', macrodatafile);
  fprintf(1, '%s\n', yielddatafile);
  fprintf(1, '%s\n\n', maturitydatafile);
  if (numloops == 1) 
    fprintf(1, ['A single set of starting parameters is used to ', ...
                'begin the optimization.\n']);
  end
  if (numloops > 1) 
    fprintf(1, ['%d sets of starting parameters are used to ', ...
                'begin the optimization.\n'], numloops);
    fprintf(1, 'All but the first are randomly generated.\n'); 
  end
  if (writetemp_copy == 1) 
    fprintf(1, ['Temporary results are written to tempparams.out ', ...
                'and tempindex.out.\n']); 
  else
    fprintf(1, 'Temporary results are not written anywhere.\n'); 
  end
  fprintf(1, '\n'); 
end

[free_estimates all_estimates loglike exitflag grad hessian] = ...
    solve_ml( use_latent, shortrate_index, freeparams, ...
              allparams, free_index, macrodata, ...
              yielddata, maturities, n, numbonds, periodlength, ...
              numloops, numsimplex, numderiv, stddevs, printscreen, ...
              writetemp_copy, restrict_unconmean, ...
              mineigen_allowed, kalman_tolerance, ...
              display_simplex, display_deriv, restart_index); 

T = size(yielddata,1);
mean_hessian = hessian/T; 
[funval funderiv loglikes loglike_derivs] = ...
    kalman_only_loglike(free_estimates, ...
                        all_estimates, free_index, ...
                        macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, restrict_unconmean, ...
                        mineigen_allowed); 
S_matrix = (1/T)* loglike_derivs' * loglike_derivs; 

[inv_hessian ok_hessian] = invpd(mean_hessian); 
if ((ok_hessian == 1) & (size(inv_hessian,1) == size(S_matrix,1))) 
  robust_stderrs = sqrt((1/T)*diag(inv_hessian * S_matrix * inv_hessian));
else
  robust_stderrs = zeros(length(free_estimates), 1);
end
[invS okSinv p] = invpd(S_matrix); 
outerprod_stderrs = sqrt((1/T)*diag(invS)); 

[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
    = unpack_allparams(n, use_latent, all_estimates, numbonds); 
errorSD = abs(errorSD); 
% lower trangular sigma matrices do not necessarily have
% nonnegative values on their diagonals.  Here I make sure that
% this normalization is imposed.  
Sigma_f = zeros(n,n); 
index = 0;
for i = 1:n
  for j = 1:i
    index = index + 1;
    Sigma_f(i,j) = fund_S(index);
  end
end
Q = Sigma_f*Sigma_f';
[tmp p] = chol(Q); 
if (p == 0)
  Sigma_f = tmp'; 
  index = 0; 
  for i = 1:n
    for j = 1:i
      index = index + 1;
      fund_S(index) = Sigma_f(i,j);
    end
  end
end
% now for the term premia sigma matrix.  
if (use_latent == 0)
  prem_K = prem_K_initialval;
  prem_S = prem_S_initialval;
else
  Sigma_omega = zeros(n,n); 
  index = 0;
  for i = 1:n
    for j = 1:i
      index = index + 1;
      Sigma_omega(i,j) = prem_S(index);
    end
  end
  Q = Sigma_omega*Sigma_omega';
  [tmp p] = chol(Q); 
  if (p == 0) 
    Sigma_omega = tmp'; 
    index = 0; 
    for i = 1:n
      for j = 1:i
        index = index + 1;
        prem_S(index) = Sigma_omega(i,j);
      end
    end
  end
end

% run the code a final time to get derivatives.  I get derivatives
% for all parameters, including those that are fixed.  

[all_estimates dummy_free_index] = pack_allparams( ...
    use_latent, fund_K, fund_S, prem_K, prem_S, ...
    prem_c, prem_macro, errorSD, ...
    ones(length(fund_K),1), ...
    ones(length(fund_S),1), ones(length(prem_K),1), ...
    ones(length(prem_S),1), ones(length(prem_c),1), ...
    ones(length(prem_macro),1), ones(length(errorSD),1));
dummy_free_params = all_estimates; 

[funval funderiv loglikes loglike_derivs filtered_values] = ...
    kalman_only_loglike(dummy_free_params, ...
                        all_estimates, dummy_free_index, ...
                        macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, restrict_unconmean, ...
                        mineigen_allowed); 

score = -funderiv; 

write_output(inputfile, outputfile, n, fund_K, fund_S, prem_K, ...
             prem_S, prem_c, prem_macro, errorSD, loglike, ...
             exitflag, score, robust_stderrs, outerprod_stderrs); 

return
