function [insamp_vector outsamp_vector] = forecast_log_1yr(macrodata, ...
                yields, long_maturity, oneyear_yield, ...
                periodsperyear, R, ma_lags, newey_west)
%
% Greg Duffee
% Feb 2006, revised April 2006
%
% This routine regresses one-year excess returns to a zero-coupon
% bond with original maturity 'long_maturity' (in years) on predetermined
% 'macrodata'.  Test statistics are adjusted for various choices of
% the number of MA lags in the covariance matrix.  These choices
% are in the vector 'ma_lags'.  The vector of flags 'newey_west'
% says whether the covariance matrix should use the newey_west
% adjustment or use generalized hansen-hodrick standard errors.  
%
% 'yields' is a vector with two columns.  The rows are dates and
% the columns are yields on bonds with maturities separated by one
% year.  Annual returns to the longer-maturity bond are calculated
% using this matrix.   
%
% The routine also constructs Ericsson's test statistic for
% out-of-sample forecastability of one-year excess returns.
% Ericsson's test requires that an initial in-sample regression
% using 'R' observations be estimated, then used to form the
% initial forecast.  This is why 'R' is passed to the routine.
% 'periodsperyear' is passed so that one-year returns can be
% constructed properly.  If, say, data are monthly, the one-year
% return depends on the difference in yields across 12
% observations.  If they are quarterly, the return depends on the
% difference in yields across 4 observations.  
%
% The in-sample results are returned in the vector
% 'insamp_vector'.  First are stacked the OLS coefficients, then
% are stacked the vectors of standard errors; one vector for each
% choice of MA lag.  Then are stacked Wald tests of the hypothesis
% that the coefficients on the macro variables are jointly zero;
% one test per choice of MA lag.  Finally come the p-values of the
% test, given a chi^2 distribution. 
%
% If 'R' is set to zero, that is a flag to indicate not to perform
% the out-of-sample test.  If the test is performed, the routine
% returns 'outsamp_vector', which is detailed in the routine
% 'enc_regtest'.   

if (nargin<8)
  newey_west = ones(size(ma_lags)); 
end;

if (R > 0)
  run_outofsample = 1;
else
  run_outofsample = 0; 
end

X = macrodata(1:(end-periodsperyear),:); 

Y = yields(1:(end-periodsperyear),2)*long_maturity - ...
    yields((1+periodsperyear):end,1)*(long_maturity-1) - ...
    oneyear_yield(1:(end-periodsperyear));

T = size(Y,1); 
this_X = [ones(T,1) X]; 
k = size(this_X,2); 
R_matrix = eye(k);
R_matrix = R_matrix(2:end,:); 
[coefs temp resids] = regress(Y, this_X); 
joint_val = R_matrix * coefs; 
dim_test = length(joint_val); 
rsquare = 1 - (transpose(resids)*resids) / ...
          (transpose(Y - mean(Y)) * (Y - mean(Y))); 
    
% Now compute standard errors and test statistics for each lag
% length.  First, set up the moment matrix

moments = [];
for i = 1:k
  moments = [moments resids .* this_X(:,i)];
end

%
% Estimate of covariance matrix of contemporaneous moments.
% go through each lag length

stderrs = [];
test_stats = [];
p_vals = []; 

XprimeX_inv = invpd(transpose(this_X)*this_X); % note, uses my invpd 
S0 = (1/T) * transpose(moments) * moments; 

for lag_index = 1:length(ma_lags)
  lags = ma_lags(lag_index); 
  use_newey_west = newey_west(lag_index); 
  S = S0;
  if (lags > 0) 
    for j = 1:lags
      F.matrix = (1/T) * transpose(moments(1:(end-j),:)) * ...
          moments((1+j):end,:); 
      if (use_newey_west == 1)
        S = S + (1.0 - (j/(lags+1.0))) * (F.matrix + transpose(F.matrix)); 
      else
        S = S + (F.matrix + transpose(F.matrix)); 
      end
    end
  end
  vcv = T * XprimeX_inv * S * XprimeX_inv; 
  [invmat okinverse] = invpd(R_matrix * vcv * transpose(R_matrix));
  if (okinverse ~= 1) % not positive definite, then redo, using
                      % newey-west.  This is only relevant if
                      % lags > 0.  
    S = S0;
    for j = 1:lags
      F.matrix = (1/T) * transpose(moments(1:(end-j),:)) * ...
          moments((1+j):end,:); 
      S = S + (1.0 - (j/(lags+1.0))) * (F.matrix + transpose(F.matrix)); 
    end
    vcv = T * XprimeX_inv * S * XprimeX_inv; 
    [invmat okinverse] = invpd(R_matrix * vcv * transpose(R_matrix));
  end
  joint_test = transpose(joint_val) * invmat * joint_val; 
  joint_pval = 1.0 - chi2cdf(joint_test, dim_test); 
  these_stderrs = sqrt(diag(vcv)); 
  stderrs = [stderrs; these_stderrs]; 
  test_stats = [test_stats; joint_test]; 
  p_vals = [p_vals; joint_pval]; 
  
end

insamp_vector = [coefs; stderrs; test_stats; p_vals]'; 

if (run_outofsample == 1) 
  overlap_length = periodsperyear - 1; 
  outsamp_vector = enc_regtest(Y, X, R, overlap_length, ma_lags, ...
                               newey_west)'; 
else
  outsamp_vector = [];
end
  
return

