function results_vector = forecast_outofsamp(macrodata, excess_rets, ...
                       R, horizons, ma_lag_matrix, newey_west)
%
% Greg Duffee
% February 2006
%
% This routine calculates Ericsson (1992) t-tests of out-of-sample
% predictive power of 'macrodata' for k-horizon excess returns.
%
% The series 'excess_rets' contains one-period excess returns.
% k-horizon returns are constructed by summing the single-period
% excess returns.  When passed, 'macrodata' must already be lagged
% once relative to excess_rets.  
%
% For a given k, the covariance matrix of parameter estimates is
% estimated using various MA lags.  The matrix 'ma_lag_matrix'
% contains the number of MA lags used.  Each row corresponds to a
% different k.  The number of columns is the number of different MA
% lags used for each out-of-sample test.  For each horizon and MA lag,
% there is a flag in the matrix 'newey_west' that indicates whether
% the standard errors use the newey-west procedure or whether the
% robust hansen-hodrick procedure is used.  
%
% The output 'results_vector' is a vector of stacked results from
% enc_regtest; one stack per horizon k.  See enc_regtest for a
% discussion of the output.  
% 

% The default is to use Newey-West...
if (nargin<6); newey_west = ones(size(ma_lag_matrix)); end;

total_obs = size(macrodata,1); 

%
% set up regressions.  Over what horizon are returns constructed?  
%

num_horizons = length(horizons); 
results_vector = []; 

for horizon_index = 1:num_horizons
  this_horizon = horizons(horizon_index); 
  these_lags = ma_lag_matrix(horizon_index,:); 
  this_newey_west = newey_west(horizon_index,:); 
  
  X = macrodata(1:(end-this_horizon+1),:); 
  Y = zeros(total_obs - this_horizon + 1, 1); 
  for i = 1:this_horizon
    Y = Y + excess_rets(i:(end-this_horizon+i)); 
  end

  enc_results = enc_regtest(Y, X, R, (this_horizon - 1), these_lags, ...
                            this_newey_west); 

  results_vector = [results_vector; enc_results]; 
end

results_vector = results_vector';

return
