function [funval loglikes filtered_values] = ...
    kalman_only_loglike_noderiv(freeparams, ...
                        allparams, free_index, macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, kalman_tol, restrict_unconmean, ...
                                mineigen_allowed)
% KALMAN_ONLY_LOGLIKE returns the (negative of the) Kalman filter
% log-likelihood.  This is used for standard ML estimation.  The
% code allows a restriction placed on the smallest (most negative) 
% eigenvalue of the equivalent-martingale feedback matrix.  If this 
% eigenvalue is less than -1, such as -1.1, the mean term
% structure has an unpleasant explosive zigzag shape.  


%
% Greg Duffee
% April 2005, revised October 2005, September 2006
%

% The model assumes three fundamental factors, so this parameter is
% hardwired here.

n = 3; 
numbonds = length(maturities);

% assume no restriction on eigenvalues if there is no passed
% argument

if (nargin<12); mineigen_allowed = -1e10; end;

paramvec = poke_freeparams(freeparams, allparams, free_index); 
[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
              = unpack_allparams(n, use_latent, paramvec, numbonds); 
[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 
%
% set mu_f
%

macroMean = mean(macrodata)'; 
mu_f = (eye(n) - K_f) * macroMean;

[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 

% check on stationarity of VAR for all factors (both macro and
% term premia factors).  If nonstationary, bail out.  
physical_eigenvals = eig(K_x); 
if (max( physical_eigenvals .* conj(physical_eigenvals)) >= 1)
  loglikes = -1e10;
  filtered_values = []; 
else
  % check if equivalent-martingale dynamics have admissible
  % eigenvalues.  This is determined by 'mineigen_allowed'.  
  % Also check whether the mean yield curve has reasonable
  % properties. 
  eigens = eig(K_x_equiv);
  real_eigens = real(eigens);
  imag_eigens = imag(eigens); 
  min_imag_eigen = min(imag_eigens); 
  signed_abs_eigens = sign(real(eigens)) .* abs(eigens); 
  min_signed_abs_eigen = min(signed_abs_eigens); 
  mineigen = min(signed_abs_eigens);
  maxYield = 1.5*max(mean(yielddata)); 
  maxSlope = 2.0*(max(mean(yielddata)) - mean(macrodata(:,3)));
  bigmats = [1; maturities];
  [Avec Bmat] = get_yield_mapping(shortrate_index, ...
                                        mu_x_equiv, K_x_equiv, Sigma_x, ...
                                        bigmats, periodlength); 
  uncon_mean_x = inv(eye(length(mu_x)) - K_x) * mu_x;
  uncon_mean_yields = Avec + Bmat * uncon_mean_x; 
  min_uncon_mean = min(uncon_mean_yields);
  max_uncon_mean = max(uncon_mean_yields);
  min_slope = min(uncon_mean_yields(2:end)) - uncon_mean_yields(1);
  max_slope = max(uncon_mean_yields(2:end)) - uncon_mean_yields(1);
  % If inadmissible, bail out.  
  if restrict_unconmean 
    invalid = ...
        (min(min_imag_eigen, min_signed_abs_eigen) < mineigen_allowed) ...
        | (min_uncon_mean < 0) | (max_uncon_mean > maxYield) | ...
        (min_slope < -0.02) | (max_slope > maxSlope);
  else
    invalid = (min(min_imag_eigen, min_signed_abs_eigen) < ...
               mineigen_allowed); 
  end
  if invalid
    loglikes = -1e10;
    filtered_values = []; 
  else
    % If model is stationary and equiv dynamics pass the filter on
    % admissible eigenvalues, run the Kalman filter routine.  To do
    % this, get the pricing functions, then get the first two
    % unconditional moments of the VAR system.  
    [Avec Bmat B_pricing_mat] = ...
        get_yield_mapping(shortrate_index, ...
                          mu_x_equiv, K_x_equiv, Sigma_x, ...
                          maturities, periodlength); 
    [A B H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 
    [uncon_mean uncon_vcv] = ...
        initial_forecast(use_latent, numbonds, mu_f, K_f, ...
                         Sigma_f, K_omega, Sigma_omega); 
    [errors loglikes filtered_values cond_vars ...
     contemp_vars] = ...
        kalman_recursion_fast( ...
            [macrodata yielddata], mu_x, K_x, Sigma_x, ...
                         A, B, H, uncon_mean, uncon_vcv, kalman_tol);
  end
end

%sum(loglikes)

funval = -sum(loglikes);

return
