function [A_derivs B_derivs H_derivs] = ...
    measurement_derivs(n, use_latent, Ayield_derivs, Byield_derivs, ...
                       errorSD, fundMean)
%
% MEASUREMENT_DERIVS constructs derivatives of A, B, and H 
% for all of the parameters of the model.  Almost all of these
% derivatives are zero.
% Parameter order with latent factors, and their position in the vector
% vec(K_f)          (1 to n^2)
% vech(Sigma_f)     (n^2+1 to n^2+(n(n+1)/2))
% vec(K_omega)      (n^2+(n(n+1)/2)+1 to 2n^2+(n(n+1)/2))
% vech(Sigma_omega) (2n^2+(n(n+1)/2)+1 to 2+3n^2)
% lambda_0f         (n+3n^2+1 to 2n+3n^2)
% vec(lambda_1f)    (2n+3n^2+1 to 2n+4n^2)
% errorSD           (2n+4n^2+1 to 3n+4n^2+1)
%
% Parameter order without latent factors, and their position in the vector
% vec(K_f)          (1 to n^2)
% vech(Sigma_f)     (n^2+1 to n^2+(n(n+1)/2))
% lambda_0f         (n^2+(n(n+1)/2)+1 to n+n^2+(n(n+1)/2))
% vec(lambda_1f)    (n+n^2+(n(n+1)/2)+1 to n+2n^2+(n(n+1)/2))
% errorSD           (n+2n^2+(n(n+1)/2)+1 to 2n+2n^2+(n(n+1)/2) + 1)
%
% For reference, here is the ordering of derivatives in
% Ayield_derivs.  It is a (numbonds, num-a-params) array, where the
% row refers to the bond and the column refers to the following
% parameters of the Ayield function:
% delta_x,                 length cdim (either n or 2n, depending on 
%                          whether latent factors are included).  
% equivmu,                 length cdim.  
% vec(equivK),             length cdim^2.  
% vech(Sigma), row by row, length cdim(1+cdim)/2. 
%
% For additional reference, here is the ordering of derivatives in
% Byield_derivs.  It is a (numbonds, numfactors, num-a-params)
% array, where the first index refers to the bond, the second to
% the factor loading, and the third to the following parameters of
% the Byield function: 
% delta_x,                 length cdim
% vec(equivK)              length cdim^2 .

%
% Greg Duffee
% September 2006
%

numbonds = size(Ayield_derivs,1); 
if (use_latent == 1) 
  cdim = 2*n;
  numparams = 3*n + 4*n^2 + 1; 
else
  cdim = n; 
  numparams = 2*n + 2*n^2 + (n*(n+1)/2) + 1; 
end

rdim = n + numbonds; 

A_derivs = zeros(rdim, numparams); 
B_derivs = zeros(rdim, cdim, numparams); 
H_derivs = zeros(rdim, rdim, numparams); 

% derivatives for A.  

% first, the derivatives wrt K_f that work through mu_f. 
% start index of Ayield_derivs at cdim because that is length of
% delta_x. 
%
index = 0; 
for j = 1:n
  for i = 1:n
    index = index + 1; 
    Z = zeros(n,n);
    Z(i,j) = 1; 
    A_derivs((n+1):end,index) = Ayield_derivs(:,(cdim+1):(cdim+n)) ...
        * (-Z*fundMean);
  end
end

% The n^2 elements of vec(K_f).  I have to pick out the upper left
% quadrant of K_x, so I can't just read in n^2 elements of K_x.  I
% need to go column-by-column.  
for i = 1:n
  A_derivs((n+1):end,(n*(i-1)+1):(i*n)) = ...
  A_derivs((n+1):end,(n*(i-1)+1):(i*n)) + ...
    Ayield_derivs(:, (2*cdim+cdim*(i-1)+1):(2*cdim+cdim*(i-1)+n)); 
end
% the n(n+1)/2 elements of vech(Sigma_f).  Here I go row-by-row, so
% the upper left quadrant is automatically filled in.  
A_derivs((n+1):end,(n^2+1):(n^2+(n*(n+1)/2))) = ...
    Ayield_derivs(:, (2*cdim+cdim^2+1):(2*cdim+cdim^2+(n*(n+1)/2))); 
if (use_latent == 1)
  % the n^2 elements of K_omega. 
  start = n^2+(n*(n+1)/2); 
  for i = 1:n
    A_derivs((n+1):end,(start+n*(i-1)+1):(start+i*n)) = ...
        Ayield_derivs(:,(2*cdim+cdim*(n+i-1)+n+1):(2*cdim+(n+i)*cdim)); 
  end
  % the n(n+1)/2 elements of Sigma_omega.  Again, I am going
  % row-by-row, but because the relevant elements of Sigma_omega
  % begin in column n+1, not column 1, I will go through the
  % elements explicitly.  
  a_index = 2*n^2+(n*(n+1)/2); 
  y_index = 2*cdim+cdim^2 + (n*(n+1)/2); 
  for i = 1:n
    y_index = y_index + n; 
    for j = 1:i
      a_index = a_index + 1; 
      y_index = y_index + 1; 
      A_derivs((n+1):end,a_index) = Ayield_derivs(:,y_index);
    end
  end
end
% the n elements of lambda0f.  
if (use_latent == 1)
  start = n+3*n^2;
else
  start = n^2+(n*(n+1)/2);
end
A_derivs((n+1):end,(start+1):(start+n)) = ...
    -Ayield_derivs(:, (cdim+1):(cdim+n)); 
% the n^2 elements of lambda1f.  Just use negative of K_f derivs.  
% I have to pick out the upper left
% quadrant of K_x, so I can't just read in n^2 elements of K_x.  I
% need to go column-by-column.  
for i = 1:n
  A_derivs((n+1):end,(start+n+n*(i-1)+1):(start+n+i*n)) = ...
    -Ayield_derivs(:, (2*cdim+cdim*(i-1)+1):(2*cdim+cdim*(i-1)+n)); 
end

% derivatives for B
%
% The n^2 elements of vec(K_f).  I have to pick out the upper left
% quadrant of K_x, so I can't just read in n^2 elements of K_x.  I
% need to go column-by-column.  
for i = 1:n
  B_derivs((n+1):end,:,(n*(i-1)+1):(i*n)) = ...
    Byield_derivs(:,:,(cdim+cdim*(i-1)+1):(cdim+cdim*(i-1)+n)); 
end
if (use_latent == 1)
  % the n^2 elements of K_omega. 
  start = n^2+(n*(n+1)/2); 
  for i = 1:n
    B_derivs((n+1):end,:,(start+n*(i-1)+1):(start+i*n)) = ...
        Byield_derivs(:,:,(cdim+cdim*(n+i-1)+n+1):(cdim+(n+i)*cdim)); 
  end
end
% the n^2 elements of lambda1f.  
if (use_latent == 1)
  start = 2*n+3*n^2;
else
  start = n+n^2+(n*(n+1)/2);
end
B_derivs((n+1):end,:,(start+1):(start+n^2)) = ...
    -B_derivs((n+1):end,:,1:n^2); 

% derivatives for H.  Remember the inputs are in percentage points 

if (use_latent == 1)
  start = 2*n+4*n^2;
else
  start = n+2*n^2+(n*(n+1)/2); 
end
for i = 1:n
  H_derivs(i, i, (start+i)) = 2.0*errorSD(i)/(100.0^2); 
end
for i = (n+1):rdim
  H_derivs(i, i, (start+n+1)) = 2.0*errorSD(n+1)/(100.0^2); 
end

return
