function results_vector = multihorizon_regressions(...
    Xdata, Ydata, horizons, ma_lag_matrix, newey_west)
%
% Greg Duffee
% November 2005, revised April 2006
%
% Regressions of a summed y vector on an x matrix.
% (These regressions are designed for regressions of excess returns
% over some horizon on explanatory variables.)
%
% The dependent variable is the sum of the Y vector from
% observation t through observation t+k-1 (a k-sum).  The
% explanatory variables are in the X matrix.  Multiple values of k
% are allowed.  They are contained in the vector 'horizons'.  
%
% For a given k, the covariance matrix of parameter estimates is
% estimated using various MA lags.  The matrix 'ma_lag_matrix'
% contains the number of MA lags used.  Each row corresponds to a
% different k.  The number of columns is the number of different MA
% lags used for each regression.  For each horizon and MA lag,
% there is a flag in the matrix 'newey_west' that indicates whether
% the standard errors use the newey-west procedure or whether the
% robust hansen-hodrick procedure is used.  
%
% The output is a single vector.  (For reasons that escape me, it
% is a row vector.)  The results for each regression
% (i.e., different k's) are stacked sequentially in the vector.
% For a given regression, the first set of variables returned are
% the OLS regression coefficients.  After these coefficients comes
% the standard errors for each coefficient.  There is a set of
% standard errors for each choice of MA lag, so the number of
% standard errors is (length of coef vector)*(number of columns of
% ma_lag_matrix).  Following the standard errors are Wald tests of
% the hypothesis that the coefficients (except the constant term)
% are identically zero.  The number of Wald statistics for a single
% regression equals the number of columns of the ma_lag_matrix.
% Finally, the p-values of these statistics are reported using a
% chi^2 distribution.  There are as many p-values as there are Wald
% tests. 

% The default is to use Newey-West...
if (nargin<5); newey_west = ones(size(ma_lag_matrix)); end;

total_obs = size(Xdata,1); 
num_macro = size(Xdata,2); 

%
% set up regressions.  Over what horizon is the Y variable summed?
%

num_horizons = length(horizons); 
results_vector = []; 

for horizon_index = 1:num_horizons
  this_horizon = horizons(horizon_index); 
  these_lags = ma_lag_matrix(horizon_index,:); 
  this_newey_west = newey_west(horizon_index,:); 
  
  X = Xdata(1:(end-this_horizon+1),:); 
  Y = zeros(total_obs - this_horizon + 1, 1); 
  for i = 1:this_horizon
    Y = Y + Ydata(i:(end-this_horizon+i)); 
  end

  T = size(Y,1); 
  this_X = [ones(T,1) X]; 
  k = size(this_X,2); 
  R_matrix = eye(k);
  R_matrix = R_matrix(2:end,:); 
  [coefs temp resids] = regress(Y, this_X); 
  joint_val = R_matrix * coefs; 
  dim_test = length(joint_val); 
  rsquare = 1 - (transpose(resids)*resids) / ...
            (transpose(Y - mean(Y)) * (Y - mean(Y))); 
    
% Now compute standard errors and test statistics for each lag
% length.  First, set up the moment matrix

  moments = [];
  for i = 1:k
    moments = [moments resids .* this_X(:,i)];
  end

%
% Estimate of covariance matrix of contemporaneous moments.
% go through each lag length

  stderrs = [];
  test_stats = [];
  p_vals = []; 

  XprimeX_inv = invpd(transpose(this_X)*this_X); % note, uses my invpd 
  S0 = (1/T) * transpose(moments) * moments; 

  for lag_index = 1:length(these_lags)
    lags = these_lags(lag_index); 
    use_newey_west = this_newey_west(lag_index); 
    S = S0;
    if (lags > 0) 
      for j = 1:lags
        F.matrix = (1/T) * transpose(moments(1:(end-j),:)) * ...
            moments((1+j):end,:); 
        if (use_newey_west == 1)
          S = S + (1.0 - (j/(lags+1.0))) * (F.matrix + transpose(F.matrix)); 
        else
          S = S + (F.matrix + transpose(F.matrix)); 
        end
      end
    end
    vcv = T * XprimeX_inv * S * XprimeX_inv; 
    [invmat okinverse] = invpd(R_matrix * vcv * transpose(R_matrix));
    if (okinverse ~= 1) % if not positive definite, then redo, using
                        % newey-west.  This is only relevant if
                        % lags > 0.  
      S = S0;
      for j = 1:lags
        F.matrix = (1/T) * transpose(moments(1:(end-j),:)) * ...
            moments((1+j):end,:); 
        S = S + (1.0 - (j/(lags+1.0))) * (F.matrix + transpose(F.matrix)); 
      end
      vcv = T * XprimeX_inv * S * XprimeX_inv; 
      [invmat okinverse] = invpd(R_matrix * vcv * transpose(R_matrix));
    end
    joint_test = transpose(joint_val) * invmat * joint_val; 
    joint_pval = 1.0 - chi2cdf(joint_test, dim_test); 
    these_stderrs = sqrt(diag(vcv)); 
    stderrs = [stderrs; these_stderrs]; 
    test_stats = [test_stats; joint_test]; 
    p_vals = [p_vals; joint_pval]; 
  
  end

  these_results = [coefs; stderrs; test_stats; p_vals]; 
  results_vector = [results_vector these_results'];

end

return
