function [simple_insamp_results simple_outsamp_results ...
         log_insamp_results log_outsamp_results] = ...
    sim_regressions( ...
    outputfile, shutoff_latent, numsims, sim_length, init_simseed, ...
    used_macro_indexes, simple_newey_west, log_newey_west, ...
    init_mat, simple_horizons, simple_ma_lag_matrix, ...
    log_ma_lag_vector, insamp_length)
%
% Greg Duffee
% February 2006, revised April 2006, June 2007
% 
% This big routine does all of the simulations of return
% forecasting regressions (both in-sample and out-of-sample) under
% the general null hypothesis.  Two types of returns are used
% here.  They are simple quarterly returns and log annual returns.
% Excess quarterly returns are constructed by subtracting the
% return to a one-quarter bond and excess annual returns are
% contructed by subtracting the yield on a one-year bond.  
%
% To complicate matters a bit, this routine does something
% that the paper does not discuss -- it allows for forecasting
% regressions using k-horizon sums of simple returns.  I did this
% to check whether the differences between the use of simple
% quarterly and log annual were entirely driven by quarterly versus
% annual, or whether simple versus log had something to do with
% it.  (The answer -- simple versus log has nothing to do with it.)
% Thus the input and output of this routine are slightly more
% complicated than you might think.  
%
% The parameters of the term structure model used to simulate under
% the general null is in 'outputfile'.  'shutoff_latent' determines
% whether any term premia factors are included.  If it is one, they
% are excluded.  The number of simulations and the length
% of each simulation are specified in 'numsims and 'sim_length'.
% The initial seed for the random number generator is
% 'init_simseed'. 
%
% The macro variables used to forecast are specified in
% 'used_macro_indexes', which is a vector containing some
% combination of (1,2,3).  The initial maturity of the bond for
% which excess returns are calculated is 'init_mat'.  This maturity
% is in quarters.  It must be at least 5, because of the definition
% of annual log excess returns (subtracting the one-year yield).  
% The horizons over which simple returns are summed are in
% 'simple_horizons'.   For a given horizon, the standard errors of
% the regression coefficients can be estimated using various MA
% lags.  The matrix 'simple_ma_lag_matrix' contains the number of
% MA lags used. Each row corresponds to a different holding period
% horizon.  The number of columns is the number of different MA
% lags used for each regression.  For each horizon and MA lag,
% there is a flag in the matrix 'simple_newey_west' that indicates
% whether the standard errors use the newey-west procedure or
% whether the robust hansen-hodrick procedure is used.  
%
% 'log_ma_lag_vector' and 'log_newey_west' play the same role in
% the annual excess return regressions, but they are vectors
% instead of matrices because only one holding period length is
% considered.  
%
% The out-of-sample test sets aside the first 'insamp_length'
% observations to estimate the initial set of parameters.  If this
% number is zero, that is a flag to tell the routine not to conduct
% any out-of-sample tests.   
%
% The output consists of four matrices (as long as the
% out-of-sample tests are performed).  Each matrix has 'numsims'
% rows, one for each simulation.  The rows for the annual log
% returns are described in 'forecast_log_1yr'.  The rows for the
% in-sample simple returns are described in
% 'multihorizon_regressions' and the rows for the out-of-sample
% simple returns are described in 'forecast_outofsamp'.  

if (insamp_length > 0)
  run_outofsample = 1;
else
  run_outofsample = 0; 
end

%
% get matrices for running simple regression simulations.
%

sim_maturities = [(init_mat - 1); init_mat]; 
sim_numbonds = length(sim_maturities); 
[c T R A B chol_H uncon_mean chol_initial_var ...
 shortrate_index periodlength] = ...
    setup_sim(outputfile, sim_maturities, shutoff_latent); 

%
% run simple regression simulations
%

simple_insamp_results = []; 
simple_outsamp_results = []; 
simseed = init_simseed; 
for i = 1:numsims
    [simdata newseed state_matrix] = sim_data(c, T, R, A, B, chol_H, ...
    uncon_mean, chol_initial_var, sim_length, simseed); 
    simseed = newseed;
    macro = simdata(:, 1:3); 
    lagged_macro = macro(1:(end-1),used_macro_indexes); 
    sim_yielddata = simdata(:,(end-sim_numbonds+1):end);
    shortret = exp(periodlength*simdata(1:(end-1),shortrate_index)); 
    sim_excessrets = [];
    longret = exp( periodlength*( ...
        sim_yielddata(1:(end-1),2)*sim_maturities(2) - ...
        sim_yielddata(2:end,1)*sim_maturities(1) ) ); 
    sim_excessrets = longret - shortret; 
    insamp_results = multihorizon_regressions(lagged_macro, sim_excessrets, ...
             simple_horizons, simple_ma_lag_matrix, ...
                                              simple_newey_west);
    simple_insamp_results = [simple_insamp_results; insamp_results]; 
    if (run_outofsample == 1) 
      outsamp_results = ...
          forecast_outofsamp(lagged_macro, sim_excessrets, ...
                             insamp_length, simple_horizons, ...
                             simple_ma_lag_matrix, simple_newey_west);
      simple_outsamp_results = [simple_outsamp_results; outsamp_results]; 
    end
end

%
% get matrices for running overlapping log regression simulations.
%

% I need maturities of one year (for the one-year yield),
% 'init_mat' (for the yield on the original bond), and 'init_mat -
% 4' (for the yield on the original bond one year later).  
%

if (init_mat == 8) 
  sim_maturities = [(init_mat - 4); init_mat]; 
else
  sim_maturities = [4; (init_mat -4); init_mat]; 
end
sim_numbonds = length(sim_maturities); 
[c T R A B chol_H uncon_mean chol_initial_var ...
 shortrate_index periodlength] = ...
    setup_sim(outputfile, sim_maturities, shutoff_latent); 
year_maturity = round(init_mat/4); 
%
% run log regression simulations
%

log_insamp_results = []; 
log_outsamp_results  = []; 
simseed = init_simseed; 
for i = 1:numsims
    [simdata newseed] = sim_data(c, T, R, A, B, chol_H, ...
    uncon_mean, chol_initial_var, sim_length, simseed); 
    simseed = newseed;
    macro = simdata(:, 1:3); 
    sim_macrodata = macro(:,used_macro_indexes);
    oneyear_yield = simdata(:,(end-sim_numbonds+1)); 
    sim_yielddata = simdata(:,(end-1):end);
    [insamp_vector outsamp_vector] = forecast_log_1yr(sim_macrodata, ...
             sim_yielddata, year_maturity, oneyear_yield, ...
             4, insamp_length, log_ma_lag_vector, ...
                        log_newey_west); 
    log_insamp_results = [log_insamp_results; insamp_vector]; 
    log_outsamp_results = [log_outsamp_results; outsamp_vector]; 
end


return
