function [error loglike contemp_expect contemp_var ...
          forecast_expect forecast_var partial_loglike ...
         partial_forecast_expect partial_forecast_var] = ...
    single_recursion(y, cond_expect, cond_var, A, B, H, mu, D, Q, ...
                     partial_cond_expect, partial_cond_var, partial_A, ...
                     partial_B, partial_H, ...
                     partial_mu, partial_D, partial_Q)
% SINGLE_RECURSION constructs one step of the Kalman recursion.
% The model is 
% y = A + B \alpha_t + e_t, Var(e_t) = H
% \alpha_t = mu + D f_{t-1} + u_t, var(u_t) = Q
% This routine takes as input y_t, the one-step-ahead prediction of 
% \alpha_t ('cond_expect'), and the one-step-ahead covariance of \alpha_t
% ('cond_var'), and produces the one-step-ahead prediction error of 
% y_t ('error'), the log-likelihood of the observation ('loglike'), 
% the current-period prediction of \alpha_t and the associated covariance
% matrix ('contemp_expect' and 'contemp_var' respectively), as well
% as the one-step-ahead prediction and covariance of \alpha_{t+1}
% ('forecast_expect' and 'forecast_var').  
%
% Derivatives of all of the matrices with respect to the model's
% parameters are also passed.  
%
% Note that the log-likelihood does not include the constant 
% term involving 2*pi.  
%
% Greg Duffee
% March 2005, revised October 2005, Jan 2006, June 2006, August 2006
%

numparams = size(partial_A,2); 
n = length(cond_expect);
numobserved = length(A); 
partial_loglike = zeros(numparams, 1);
partial_forecast_expect = zeros(n, numparams); 
partial_forecast_var = zeros(n, n, numparams); 
partial_error = zeros(numobserved, numparams); 
partial_F = zeros(numobserved, numobserved, numparams); 
partial_contemp_expect = zeros(n, numparams); 
partial_contemp_var = zeros(n, n, numparams); 

F = B*cond_var*B' + H; 
predicted_y = A + B*cond_expect;
detval = log(abs(det(F))); 
[invF okinv] = invpd(F); 
error = y - predicted_y; 
% dropping 2*pi terms from likelihood, and kludge to keep
% conditional variance from getting too close to zero (might
% exactly fit one observation; that would be bad)
if (okinv == 0)
  loglike = - 1e10;
  info_in_errors = zeros(length(error), 1); 
  contemp_expect = cond_expect + cond_var * B'* info_in_errors;
  contemp_var = cond_var;
else
  loglike = -0.5*(detval + error' * invF * error);
  info_in_errors = invF * error; 
  contemp_expect = cond_expect + cond_var * B' * info_in_errors;
  contemp_var = cond_var - cond_var * B'*invF*B * cond_var; 
  for i = 1:numparams
    partial_error(:,i) = -B*partial_cond_expect(:,i) - ...
        partial_B(:,:,i)*cond_expect - partial_A(:,i); 
    partial_F(:,:,i) = partial_B(:,:,i)*cond_var*B' + ...
        B*partial_cond_var(:,:,i)*B' + ...
        B*cond_var*partial_B(:,:,i)' + partial_H(:,:,i); 
    partial_loglike(i) = -0.5*( ...
        sum(diag(invF*partial_F(:,:,i)* ...
                 (eye(numobserved) - invF*error*error'))) ...
        + 2.0*partial_error(:,i)'*invF*error); 
    partial_contemp_expect(:,i) = partial_cond_expect(:,i) + ...
        partial_cond_var(:,:,i)*B'*invF*error + ...
        cond_var*partial_B(:,:,i)'*invF*error - ...
        cond_var*B'*invF*partial_F(:,:,i)*invF*error + ...
        cond_var*B'*invF*partial_error(:,i); 
    partial_contemp_var(:,:,i) = partial_cond_var(:,:,i) - ...
        partial_cond_var(:,:,i)*B'*invF*B*cond_var - ...
        cond_var*partial_B(:,:,i)'*invF*B*cond_var + ...
        cond_var*B'*invF*partial_F(:,:,i)*invF*B*cond_var - ...
        cond_var*B'*invF*partial_B(:,:,i)*cond_var - ...
        cond_var*B'*invF*B*partial_cond_var(:,:,i); 
    partial_forecast_expect(:,i) = partial_D(:,:,i)*contemp_expect ...
        + D*partial_contemp_expect(:,i) + partial_mu(:,i); 
    partial_forecast_var(:,:,i) = partial_D(:,:,i)*contemp_var*D' + ...
        D*partial_contemp_var(:,:,i)*D' + ...
        D*contemp_var*partial_D(:,:,i)' + partial_Q(:,:,i); 
  end
end
forecast_expect = mu + D*contemp_expect;
forecast_var = D*contemp_var*D' + Q; 

return

