function [error loglike contemp_expect contemp_var ...
          forecast_expect forecast_var, F] = ...
    single_recursion_noderiv(y, cond_expect, cond_var, A, B, H, mu, D, Q)
% SINGLE_RECURSION_NODERIV constructs one step of the Kalman recursion.
% The model is 
% y = A + B \alpha_t + e_t, Var(e_t) = H
% \alpha_t = mu + D f_{t-1} + u_t, var(u_t) = Q
% This routine takes as input y_t, the one-step-ahead prediction of 
% \alpha_t ('cond_expect'), and the one-step-ahead covariance of \alpha_t
% ('cond_var'), and produces the one-step-ahead prediction error of 
% y_t ('error'), the log-likelihood of the observation ('loglike'), 
% the current-period prediction of \alpha_t and the associated covariance
% matrix ('contemp_expect' and 'contemp_var' respectively), as well
% as the one-step-ahead prediction and covariance of \alpha_{t+1}
% ('forecast_expect' and 'forecast_var').  
%
% Note that the log-likelihood does not include the constant 
% term involving 2*pi.  
%
% Greg Duffee
% March 2005, revised October 2005, Jan 2006, June 2006, August 2006
%

n = length(cond_expect);
numobserved = length(A); 

F = B*cond_var*B' + H; 
predicted_y = A + B*cond_expect;
detval = log(abs(det(F))); 
[invF okinv] = invpd(F); 
error = y - predicted_y; 
% dropping 2*pi terms from likelihood, and kludge to keep
% conditional variance from getting too close to zero (might
% exactly fit one observation; that would be bad)
if (okinv == 0)
  loglike = - 1e10;
  info_in_errors = zeros(length(error), 1); 
  contemp_expect = cond_expect + cond_var * B'* info_in_errors;
  contemp_var = cond_var;
else
  loglike = -0.5*(detval + error' * invF * error);
  info_in_errors = invF * error; 
  contemp_expect = cond_expect + cond_var * B' * info_in_errors;
  contemp_var = cond_var - cond_var * B'*invF*B * cond_var; 
end
forecast_expect = mu + D*contemp_expect;
forecast_var = D*contemp_var*D' + Q; 

return

