function [mu_derivs K_derivs vcv_derivs] = ...
    transition_derivs(mu, K, Sigma, use_latent, numbonds, ...
                      fundMean)
% TRANSITION_DERIVS constructs derivatives of mu, K, and Sigma*Sigma'
% for all of the parameters of the model.  Here, mu, K, and Sigma
% refer to mu_x, K_x, and Sigma_x, which are the parameters
% associated with the joint model of fundamental factors and term
% premia factors.
%
% Almost all of these derivatives are zero.
%
% Parameter order with latent factors, and their position in the vector
% vec(K_f)          (1 to n^2)
% vech(Sigma_f)     (n^2+1 to n^2+(n(n+1)/2))
% vec(K_omega)      (n^2+(n(n+1)/2)+1 to 2n^2+(n(n+1)/2))
% vech(Sigma_omega) (2n^2+(n(n+1)/2)+1 to 2+3n^2)
% lambda_0f         (n+3n^2+1 to 2n+3n^2)
% vec(lambda_1f)    (2n+3n^2+1 to 2n+4n^2)
% errorSD           (2n+4n^2+1 to 3n+4n^2+1)
%
% Parameter order without latent factors, and their position in the vector
% vec(K_f)          (1 to n^2)
% vech(Sigma_f)     (n^2+1 to n^2+(n(n+1)/2))
% lambda_0f         (n^2+(n(n+1)/2)+1 to n+n^2+(n(n+1)/2))
% vec(lambda_1f)    (n+n^2+(n(n+1)/2)+1 to n+2n^2+(n(n+1)/2))
% errorSD           (n+2n^2+(n(n+1)/2)+1 to 2n+2n^2+(n(n+1)/2) + 1)
%

%
% Greg Duffee
% September 2006, revised April 2007
%

if (use_latent == 1) 
  n = length(mu)/2;
  numparams = 3*n + 4*n^2 + 1; 
else
  n = length(mu); 
  numparams = 2*n + 2*n^2 + (n*(n+1)/2) + 1; 
end

dim = length(mu); 
mu_derivs = zeros(dim, numparams); 
K_derivs = zeros(dim, dim, numparams); 
vcv_derivs = zeros(dim, dim, numparams); 

% derivatives of mu_x wrt K_f.  Only fundamental components are
% affected.  Parameters other than K_f are irrelevant.  

index = 0; 
for j = 1:n
  for i = 1:n
    index = index + 1; 
    Z = zeros(n,n);
    Z(i,j) = 1; 
    mu_derivs(1:n,index) = - Z*fundMean;
  end
end

% derivatives of K_x wrt elements of K_f.  
for j = 1:n
  for i = 1:n
    K_derivs(i,j,(j-1)*n+i) = 1.0;
  end
end

if (use_latent == 1)
  % derivatives of K_x wrt elements of K_omega.  
  for j = 1:n
    for i = 1:n
      K_derivs(n+i,n+j,n^2+(n*(n+1)/2) + (j-1)*n+i) = 1.0;
    end
  end
end
  
% derivatives of Sigma_x wrt the elements of Sigma_f

index = n^2;
for i = 1:n
  for j = 1:i
    index = index + 1; 
    Z = zeros(n,n);
    Z(i,j) = 1; 
    vcv_derivs(1:n,1:n,index) = (Z*Sigma(1:n,1:n)' + Sigma(1:n,1:n)*Z');
  end
end

if (use_latent == 1)
  % derivatives of Sigma_x wrt elements of Sigma_omega.  
  index = 2*n^2 + (n*(n+1)/2);
  for i = 1:n
    for j = 1:i
      index = index + 1; 
      Z = zeros(n,n);
      Z(i,j) = 1; 
      vcv_derivs((n+1):(2*n),(n+1):(2*n),index) = ...
          (Z*Sigma((n+1):(2*n),(n+1):(2*n))' + ...
           Sigma((n+1):(2*n),(n+1):(2*n))*Z');
    end
  end
end

return
