function [mean vcv stationary mean_derivs vcv_derivs] = ...
    uncon_moments(mu, K, Sigma)

% UNCON_MOMENTS constructs the unconditional mean and unconditional
% variance matrix for a VAR(1).  If the model is nonstationary, the
% flag 'stationary' is set to one and 'vcv' is empty.  
% The function also constructs derivatives of
% these unconditional moments with respect to the following
% parameter vector:
%
% mu
% vec(K) 
% vech(Sigma) [row-by-row, not column-by-column].
%
%
% Greg Duffee
% August 2006
%

n = length(mu); 
numparams = n + n^2 + n*(n+1)/2; 

eigvals = eig(K); 
if (max( eigvals .* conj(eigvals)) >= 1)
  mean = [];
  vcv = []; 
  mean_derivs = [];
  vcv_derivs = []; 
  stationary = 0; 
else
  stationary = 1; 
  invmatrix = inv(eye(n) - K); 
  Q = Sigma*Sigma'; 
  mean = invmatrix * mu;
  vec_sigma = inv( eye(prod(size(K))) - kron(K,K) ) * vec(Q);
  vcv = vec_inverse(vec_sigma);
  if (nargout > 3) 
    mean_derivs = zeros(n, numparams);
    vcv_derivs = zeros(n, n, numparams);
    % derivatives of mu
    for i = 1:n
      z = zeros(n,1);
      z(i) = 1;
      mean_derivs(:,i) = invmatrix*z;
    end
    for j = 1:n
      for i = 1:n
        Z = zeros(n,n);
        Z(i,j) = 1.0;
        mean_derivs(:,(j*n+i)) = invmatrix*Z*invmatrix*mu; 
      end
    end
    % derivatives of unconditional VCV
    big_invmatrix = inv( eye(prod(size(K))) - kron(K, K) ); 
    for j = 1:n
      for i = 1:n
        Z = zeros(n,n);
        Z(i,j) = 1.0; 
        vecderiv = big_invmatrix*(kron(Z, K) + ...
                                  kron(K, Z))*big_invmatrix*vec(Q);
        vcv_derivs(:,:,(j*n+i)) = vec_inverse(vecderiv); 
      end
    end
    index = 0;
    for i = 1:n
      for j = 1:i
        index = index + 1; 
        Z = zeros(n,n);
        Z(i,j) = 1.0;
        vecderiv = big_invmatrix * vec(Sigma*Z' + Z*Sigma');
        vcv_derivs(:,:,(n+n^2+index)) = vec_inverse(vecderiv);
      end
    end
  end
end
return
